using System;
using System.Collections;
using System.Configuration;
using System.Data.OleDb;
using System.Data;
using AdhiC.db;

namespace AdhiC
{



	/**
	 * The Data Access Object.  This class connects to the data
	 * server and executes queries. 
	 * @author DKRISHNA
	 *
	 */
	[Serializable]
	public abstract class DataAccessObject
	{
		private AdhiDBConnection connection;

			public AdhiDBConnection Connect()
			{
				return Connect(null,null);
			}

			public AdhiDBConnection Connect(string dbType) 
			{
				return Connect(dbType, null);
			}
			public AdhiDBConnection Connect(string dbType,string connString) 
			{

				if( connection==null)
				{
					CreateConnection(dbType, connString);
				}
				if (connection.getState() != ConnectionState.Open)
				{
					connection.Open();
					//Insert code to process data.
				}
				return connection;
			}
		  //execute a non query i.e., INSERT, UPDATE, DELETE statements
        public int ExecuteNonQuery(string cmdStr)
		{
            return ExecuteNonQuery(cmdStr,null,null);
		}

		public int ExecuteNonQuery(string insertStr,string dbType,string connString )
		{

            int ret = 0;
			try
			{
				AdhiDBCommand insertCMD = CreateCommand(insertStr,dbType, connString);
				Connect(dbType, connString);
				ret = insertCMD.ExecuteNonQuery();
			}
			catch(Exception e)
			{
				throw new DataException(e.Message);
			}
            finally
			{
				Close();
			}
			return ret;
	    }

		public int GetIDAfterInsert(string insertStr )
		{
			return GetIDAfterInsert(insertStr, null,null);
		}
        public int GetIDAfterInsert(string insertStr,string dbType,string connString)
		{
            int ret = 0;
            try
			{
                AdhiDBCommand insertCMD = CreateCommand(insertStr,dbType, connString);
                Connect(dbType, connString);
                insertCMD.ExecuteNonQuery();
                insertCMD.SetCommandText("SELECT @@identity");
				ret = System.Convert.ToInt32(insertCMD.ExecuteScalar());
			}
            catch(Exception ex )
			{
				throw ex;
			}
            finally
			{
				Close();
			}
				return ret;
		}

		/// <summary>
		/// Execute selection Query and return as dataset 
		/// </summary>
		/// <param name="selectStr">selectQuery</param>
		/// <returns>DataSet</returns>
		public DataSet Find(string selectStr)
		{
			return Find(selectStr, null, null);
		}
		public DataSet SPFind(string selectSPName, string[] variables,object[] values)
		{
			return SPFind(selectSPName,variables,values,null, null);
		}
		public DataSet SPFind(string selectSPName, string[] variables,object[] values,string dbType,string connString)
		{
			DataSet table  = new DataSet();
			try
			{
				AdhiDBCommand dsCmd = CreateCommand(selectSPName,dbType, connString);
				dsCmd.SetCommandType(CommandType.StoredProcedure);
				dsCmd.SetParameters(variables,values);
				Connect(dbType, connString);
				table = dsCmd.Find();
			}
			catch(Exception e)
			{
				throw e;
			}

			finally
			{
				//always call Close when done reading.
				Close();
			}
			return table;
			
		}
        
		public DataSet Find(string selectStr,string dbType,string connString)
		{

            DataSet table = null;
			try
			{
				AdhiDBCommand dsCmd = CreateCommand(selectStr,dbType, connString);
				dsCmd.SetCommandType(CommandType.Text);
				Connect(dbType, connString);
				table = dsCmd.Find();
			}
			catch(Exception e)
			{
				throw e;
			}

			finally
			{
				//always call Close when done reading.
				Close();
			}
			return table;
		}


		public void Close()
		{
			connection.Close();
		}

		private AdhiDBCommand CreateCommand(string cmdStr,string dbType,string connString)
		{
            CreateConnection(dbType, connString);
            AdhiDBCommand command ;
            if (dbType == null)
			{
				command = AdhiDBCommand.getInstance(cmdStr, connection);
			}
            else
			{
				command = AdhiDBCommand.getInstance(dbType, cmdStr, connection);
			}
				return command;
		}
		private void CreateConnection(string dbType,string connString)
		{
			if(dbType == null)
			{
				connection = AdhiDBConnection.getInstance();
			}
			else
			{
				connection = AdhiDBConnection.getInstance(dbType);
			}
            
			if(connString == null) 
			{
                connString = 
                    ConfigurationManager.AppSettings[AdhiConstants.CONNECTION_URL];
			}
			//TODO: Modify the connection string and include any
			//additional required properties for your database.
			connection.setConnectionString(connString);
		}

		
	
	}
}