using System;
using System.Data;
using System.Data.Odbc;

namespace AdhiC.db
{
	/// <summary>
	/// Summary description for AdhiOdbcCommand.
	/// </summary>
	public class AdhiOdbcCommand : AdhiDBCommand
	{
		//the constructor which takes in a connection object and the
		//command string
		public AdhiOdbcCommand(string commandStr,OdbcConnection connection)
		{
			command = new OdbcCommand(commandStr, connection);
		}

		//executes a non query
		public override int ExecuteNonQuery()
		{
			int ret=0;
			if(this.command !=null) 
			{
				ret = ((OdbcCommand)this.command).ExecuteNonQuery();
			}
			return ret;
		}

		//executes a scalar function
		public override int ExecuteScalar()
		{
			int ret=0;
			if(this.command != null )
			{
				ret = System.Convert.ToInt32(((OdbcCommand)this.command).ExecuteScalar());
			}
			return ret;
		}

		//sets the command text
		public override void SetCommandText(string cmdStr)
		{
			if(this.command  != null )
			{	
				((OdbcCommand)this.command).CommandText = cmdStr;
			}
		}

		//sets the command type
		public override void SetCommandType(CommandType cmdType)
		{
			if(this.command  != null )
			{	
				((OdbcCommand)this.command).CommandType = cmdType;
			}
		}

		//sets the command parameter
		public override void SetParameters( string[] variables,object[] values)
		{
			if(this.command  != null )
			{	
				if(!variables[0].ToString().Equals (string.Empty))
				{
					// Loop through all the parameters and values
					for(int i=0; i<variables.Length; i++)
					{
						 OdbcParameter cmdParameter = new OdbcParameter(variables[i].ToString(),values[i]);
						((OdbcCommand)this.command).Parameters.Add(cmdParameter);
					}	
				}
			}
		}
		
		//executes a query and returns a rowset object
		public override DataSet Find()
		{

			DataSet table = new DataSet();
			try
			{
				if(this.command != null ) 
				{
					OdbcDataAdapter adapter = new OdbcDataAdapter(((OdbcCommand)this.command));
					adapter.Fill(table);
				}
				
			}
			catch(Exception e)
			{
				throw e;
			}
			return table;
		}

	}
}
